//*****************************************************************************************//
//**********************************VER 1.0 data:20190524**********************************//
//**********************************    author : fred    **********************************//
//*********************************** flash memory control********************************//
//*****************************************************************************************//
//void IAP_erase(u8 EESEC,u8 area)
//EESEC is the block number , block first address is 0x200*EESEC
//operate area 0:select program area 1:select data area
//
//u8	 IAP_write(u16 EEADDR,u8 EEDATA,u8 area)
//EEADDR the operate address pROM 14bit wide/EEPROM 10bit wide
//EEDATA write data to the address 
//operate area 0:select program area 1:select data area
//return 0 : operation is success 1:operation is failure
//
//u8   IAP_read(u16 EEADDR,u8 area)
//EEADDR the operate address pROM 14bit wide EEPROM 10bit wide
//operate area 0:select program area 1:select data area
//return EEDATA is the data of the address 
//**************define constant******************//

#include <CMS/CMS8S3660.H>

//*******************variable*********************//

u8 RD_IAP;

//*******************initial*********************//

void IAP_erase(u8 EESEC,u8 area);				//EESECthe block counter 
u8	 IAP_write(u16 EEADDR,u8 EEDATA,u8 area);	//IAP write
u8   IAP_read(u16 EEADDR,u8 area);				//IAP read,return the value

//	area: data area 0:select program area 1:select data area

//******************* main  *********************//

void main()
{
	IAP_erase(0,1);								//data area the first block
	IAP_write(0x112,0x05,1);					//write data to the address
	IAP_write(0x110,0xaa,1);	
	RD_IAP=IAP_read(0x110,1);					//RD_IAP:read the data write to EE

	while(1)
	{
		clrwdt();
	}
}


/*************************************************
function name: IAP_erase()
function work: erase the IAP block
input data   : EESEC is the block number , block first address is 0x200*EESEC & area 0:select program area 1:select data area
output data  : null
note		 : erase the IAP block
*************************************************/
void IAP_erase(u8 EESEC,u8 area)				//data area example(512byte one secret)
{
	
	MADRH		=	0x02*EESEC;					//IAP write/read address low byte
	MADRL		=	0x00;						//IAP write/read address high byte
	
//*********************erase**************************//
	
	MCTRL		=	(area*16+0x0D);
	//Bit5	  MERR	1:Before the programming operation begins, detect that the data 
	//				in the programming address is not "FFH" (not erased), and the writing operation is terminated immediately
	//Bit4	  MREG	flash area select bit 0:select program area 1:select data area
	//Bit3~2  MMODE1~0 00:read mode 10:write mode 11:erase mode(the entire sector of the current address)		
	//Bit0	  MSTART	1:START erase/write/read operation
	//					write 0:stop operstion 1:
		
	_nop6_();							//delay 6 nop
	
	while(MCTRL & 0x01);				//wait the erase operation complete
	
	//Note:actual program, user must be reserve the overtime function
}	
/*************************************************
function name: IAP_write()
function work: write the IAP data
input data   : 
EEADDR the operate address pROM 14bit wide/EEPROM 10bit wide
EEDATA write data to the address 
  area 0:select program area 1:select data area
output data  : 0 : operation is success 1:operation is failure
note		 : write data to the Address
*************************************************/
u8 IAP_write(u16 EEADDR,u8 EEDATA,u8 area)		//if mode right,return 0,if mode error,return 1 ;reserve function
{
	
	if((MCTRL&0x20)==0x00)          
	{
		MADRL 		= 	EEADDR;                
		MADRH 		= 	EEADDR>>8;		//address 
		MDATA 		= 	EEDATA;   		//data 

		MCTRL		=	(area*16+0x09);
				
		_nop6_();						//delay 6 nop
		
		while(MCTRL & 0x01);        
		
	}
	else
	{
		return 1;
	}
	
		return 0;

}

/*************************************************
function name: IAP_read()
function work: read the flash data
input data   : 
EEADDR the operate address pROM 14bit wide/EEPROM 10bit wide
  area 0:select program area 1:select data area
output data  : EEDATA is the data of the address
note		 : read the data of the Address
*************************************************/
u8 IAP_read(u16 EEADDR,u8 area)
{
	
	MADRL 		= EEADDR;
	MADRH 		= EEADDR>>8;	
	
	MCTRL		=	(area*16+0x01);

	_nop6_();						//delay 6 nop

	while(MCTRL & 0x01);                

	return MDATA;
 }

