//*****************************************************************************************//
//**********************************VER 1.0 data:20190524**********************************//
//**********************************    author : fred    **********************************//
//********************************** timer0  demo code   **********************************//
//********************************** reference the note  **********************************//
//*****************************************************************************************//

#include <CMS/CMS8S3660.H>

//***********define constant*********//

#define		MAIN_CLOCK_FREQ		16000000uL		//unit:Hz
#define		TM0_OF_us			6000			//timer0 overflow times unit:us
#define		divider				4
#define		TM_value			(65535-(TM0_OF_us*(MAIN_CLOCK_FREQ/(4*1000000))))

//*******************variable*********************//


//*******************initial*********************//

void Timer0_init();								//
void INT_init();								//interrupt initial
void Timer0_INT();								//toggle watch IO

//*******************MAIN*********************//

void main()
{
	
	P1TRIS		|=	0x20;						//P15 setting output
	
	P1TRIS		|=	0x10;
	P14CFG=0x05;
	
	Timer0_init();								//timer0 initinal
	INT_init();									//interrupt initinal
	
	while(1)									//main loop
	{
		clrwdt();
	}
	
}
/*************************************************
function name: Timer0_init()
function work: initial the timer0 overflow time
input data   : the constant define the fsys and the overflow time
output data  : null
note		 : setting the timer0 
*************************************************/
void Timer0_init()								//T_overflow unit:us 16 bit timer mode
{

	TMOD		|=	0x01;
	
	//timer1 for bit7~4
	//bit7 gatebit 1:enable 0:disable
	//bit6 timer/counter 0:timer 1:counter
	//bit5~4 00:mode0 13bit timer/counter 01:mode1 16bit timer/counter 
	//		 01:mode2 8bit aoto reload timer/counter mode3:stop counter
	
	//timer0 for bit3~0
	//bit3 gatebit 1:enable 0:disable
	//bit2 timer/counter 0:timer 1:counter
	//bit1~0 00:mode0 13bit timer/counter 01:mode1 16bit timer/counter 
	//		 10:mode2 8bit aoto reload timer/counter mode3:independent timer/counter
	
	CKCON		|=	0x08;
	
	//bit7~6 WDT overflow time 00:2^17*tsys 01:2^20*tsys 10:2^23*tsys 11:2^26*tsys
	//bit4:timer1 clocksource 0:fsys/12 1:fsys/4
	//bit3:timer0 clocksource 0:fsys/12 1:fsys/4


	//initial value for 16bit counter
	
	TH0			=	TM_value/256;
	TL0			=	TM_value;
	
	TCON		|=	0x10;
	
	//bit7 flag 0:timer1 not overflow 1:timer1 over flow 
	//bit6 0:timer1 disable 1:timer1 enable
	//bit5 flag 0:timer0 not overflow 1:timer0 over flow 
	//bit4 0:timer0 disable 1:timer0 enable
	//bit3~0 is for external pin interrupt
	
}
/*************************************************
function name: INT_init()
function work: initial the INT function
input data   : null
output data  : null
note		 : setting the interrupt function
*************************************************/
void INT_init()									//intterrupt initinal
{
	IE		=	0x02;
	
//Bit7	EA	Global interrupt enable bit 0:disable 1:enable
//Bit6	ES1	UART1 interrupt enable bit  0:disable 1:enable
//Bit5	ET2	Timer2 global interrupt enable bit  0:disable 1:enable
//Bit4	ES0	UART0 interrupt enable bit  0:disable 1:enable
//Bit3	ET1	Timer1 interrupt enable bit  0:disable 1:enable
//Bit2	EX1	INT1 interrupt enable bit  0:disable 1:enable
//Bit1	ET0	Timer0 interrupt enable bit  0:disable 1:enable
//Bit0	EX0	INT0 interrupt enable bit  0:disable 1:enable

	IP		=	0x00;
	
//Bit6	PS1		UART1 interrupt priority control bit 0:high priority 1:low priority
//Bit5	PT2		Timer2 interrupt priority control bit 0:high priority 1:low priority
//Bit4	PS0		UART0 interrupt priority control bit 0:high priority 1:low priority
//Bit3	PT1		Timer1 interrupt priority control bit 0:high priority 1:low priority
//Bit2	PX1		INT1 interrupt priority control bit 0:high priority 1:low priority
//Bit1	PT0		Timer0 interrupt priority control bit 0:high priority 1:low priority
//Bit0	PX0		INT0 interrupt priority control bit 0:high priority 1:low priority

	EA		=	1;		//Bit7	EA	Global interrupt enable bit 0:disable 1:enable
	
}

/*************************************************
function name: Timer0_INT()
function work: timer0 INT function
input data   : null
output data  : null
note		 : watch the P15 toggle and measure pulse is the overflow time
*************************************************/
void Timer0_INT()	interrupt	TIMER0_VECTOR
{

	TH0			=	TM_value/256;
	TL0			=	TM_value;
	
	P1_5		=	~P1_5;						//watch IO
}